from typing import Optional, Any

from pydantic import BaseModel

from solax_py_library.smart_scene.types.action import SmartSceneActionExecute


class SmartSceneExecuteResult(BaseModel):
    needLog: bool
    actionType: Optional[SmartSceneActionExecute]
    sceneInstance: Any
    actionIsSuccessful: Optional[bool]
